#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# scikit-multiflow documentation build configuration file, created by
# sphinx-quickstart on Fri Aug 11 19:45:05 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.
import os
import sys
from sphinx.domains.python import PythonDomain   # noqa
from skmultiflow._version import __version__ as skmultiflow_version    # noqa
# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
sys.path.insert(0, os.path.abspath('sphinxext'))
from github_link import make_linkcode_resolve   # noqa

# Avoid ImportErrors from autodoc
sys.path.insert(0, os.path.abspath('../src'))


# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
needs_sphinx = '2.0.1'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ["sphinx.ext.autodoc",
              'numpydoc',
              'sphinx.ext.mathjax',
              'sphinx.ext.intersphinx',
              'sphinx.ext.linkcode',
              'sphinx_copybutton'
              ]

autosummary_generate = True   # = ["index"] to skip API

# numpydoc options
numpydoc_attributes_as_param_list = True
numpydoc_show_class_members = False
numpydoc_show_inherited_class_members = False
numpydoc_class_members_toctree = False
numpydoc_xref_ignore = {'type', 'optional', 'default'}

# autodoc options
autodoc_default_options = {'members': True,
                           'inherited-members': True,
                           }
autodoc_typehints = "none"

# Skip __init__
def autodoc_skip_member_handler(app, what, name, obj, skip, options):
    if name == "__init__":
        return True

# sphinx_copybutton options
copybutton_prompt_text = ">>> "

# mathjax path
mathjax_path = "https://cdnjs.cloudflare.com/ajax/libs/mathjax/" \
               "2.7.7/MathJax.js?config=TeX-MML-AM_CHTML"

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'scikit-multiflow'
copyright = '2018 - 2020, scikit-multiflow developers (BSD License)'
author = 'J. Montiel, J. Read, A. Bifet, T. Abdessalem'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = skmultiflow_version
# The full version, including alpha/beta/rc tags.
release = skmultiflow_version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store', 'tests/*', '*demos.rst']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# html_add_permalinks = ""


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'pydata_sphinx_theme'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
    "github_url": "https://github.com/scikit-multiflow/scikit-multiflow",
    "show_prev_next": False,
    "external_links": [
        {"name": "Webpage", "url": "https://scikit-multiflow.github.io/"},
    ],
    "use_edit_page_button": False,
}

# A list of ignored prefixes for module index sorting.
modindex_common_prefix = ['skmultiflow.']

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
# html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = "_static/images/skmultiflow-logo-wide.svg"

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = "_static/images/skmultiflow-favicon.ico"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# Additional css formatting
html_css_files = [
    'css/skmultiflow.css',
]

html_context = {
  # "display_github": True,   # Add 'Edit on Github' link instead of 'View page source'
  "github_user": "scikit-multiflow",
  "github_repo": project,
  "github_version": "master",
  "doc_path": "docs/",
  # "conf_py_path": "/docs/",
  "source_suffix": source_suffix,
}

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
html_show_copyright = False

# Output file base name for HTML help builder.
htmlhelp_basename = 'scikit-multiflowdoc'

# The empty string is equivalent to '%b %d, %Y'.
html_last_updated_fmt = ""

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'scikit-multiflow.tex', 'scikit-multiflow Documentation',
     'various', 'manual'),
]

# intersphinx configuration
intersphinx_mapping = {
    'python': ('https://docs.python.org/{.major}'.format(
        sys.version_info), None),
    'numpy': ('https://docs.scipy.org/doc/numpy/', None),
    'scipy': ('https://docs.scipy.org/doc/scipy/reference', None),
    # 'matplotlib': ('https://matplotlib.org/', None),
    'sklearn': ('https://scikit-learn.org/stable', None)
}

# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'scikit-multiflow', 'scikit-multiflow Documentation',
     [author], 1)
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'scikit-multiflow', 'scikit-multiflow Documentation',
     author, 'scikit-multiflow', 'A machine learning package for streaming data in Python.',
     'Miscellaneous'),
]

# -- Options for Epub output ----------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project
epub_author = author
epub_publisher = author
epub_copyright = copyright

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A list of files that should not be packed into the epub file.
epub_exclude_files = ['search.html']


# Avoid misleading warning triggered by more than one target for cross-reference
class PatchedPythonDomain(PythonDomain):
    def resolve_xref(self, env, fromdocname, builder, typ, target, node, contnode):
        if 'refspecific' in node:
            del node['refspecific']
        return super(PatchedPythonDomain, self).resolve_xref(
            env, fromdocname, builder, typ, target, node, contnode)


def setup(sphinx):
    sphinx.add_domain(PatchedPythonDomain, override=True)
    sphinx.connect('autodoc-skip-member', autodoc_skip_member_handler)


# The following is used by sphinx.ext.linkcode to provide links to github
linkcode_resolve = make_linkcode_resolve('skmultiflow',
                                         u'https://github.com/scikit-multiflow/'
                                         'scikit-multiflow/blob/{revision}/src/'
                                         '{package}/{path}#L{lineno}')
